// honyaku (e2j) program 
// honyaku.c

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include "macro.h"

char *dir = "/usr/home/murase/c/yak/";

FILE *fpin, *fpout, *prcs, *dbg, *wdr;
extern FILE *ejd;
char buf[SentLen+20000], path[170], path_in[170], path2[170];
extern char aj[WordNumb][YakugoMax][JWordLen], az[WordNumb][YakugoMax][ZokuLen];
extern char caj[YakugoMax][JWordLen], caz[YakugoMax][ZokuLen];
extern char en[WordNumb][SentLen], wd[WordLen], zkc, Cap;
/*char in[WordNumb][SentLen], wp[WordLen], Cmp;*/
char wp[WordLen], Cmp, Dbg = 'N';
char pwd[WordLen], pst[SentLen], sj[80], oj[80], vr[JWordLen*2+2], sz[30], oz[30], vz[30];
extern int wc, wnb, ct, quo;
char jps[JSentLen+20000], egs[SentLen+20000], egsdm[20], wdh[SentLen], wdhdm[20];
char wdx[WordLen], wdw[SentLen], wdwdm[20], wdp[WordLen];
char BOTH = 'N', LOG = 'N';
int  cnb, wi, no, dbgsz;
int  html_f = 0, arrw = 0, script = 0, style = 0, CMNT = 0;
char dics[26][27] = {	"bcdefghilmnprstuvwx ","aeeeiloruy          ",
			"aehilooru           ","aeioruy             ",
			"acdefgilmnqrsuvxy   ","aeiloru             ",
			"aehiloruy           ","aaaeiouy            ",
			"cdflmnnnorst        ","aeiou               ",
			"aeino               ","aeiou               ",
			"aaeiouy             ","aeiouy              ",
			"abcdfhiklmnprstuvwxy","aehilnoruy          ",
			"u                   ","aehiou              ",
			"acehiklmnopqtuwy    ","aehhioruwy          ",
			"fglmnprst           ","aeiou               ",
			"aehiory             ","ey                  ",
			"aeiou               ","eio                 " };
int  dic[26] = {0,20,31,41,49,67,75,85,94,107,113,119,125,133,140,161,
	172,174,181,198,209,219,225,233,236,242};


main(argc, argv)
int  argc;
char *argv[];
{
    char filnam[50], obj[50], ejdic[50], process[50], debug[50];
    int existence = 0;

    strcpy(obj, "OBJECT");
//    sprintf(obj, "%sOBJECT", dir);
    if(argc == 1){
      color_red;
      printf("This program reads an input file, containing a English\n");
      printf("sentence, such as, for example, \"I am a boy.\"\n\n");
      color_white;
      printf("Name of input file: ");	scanf("%s", filnam);
      BOTH = 'Y';    LOG = 'Y';
    }

    if(argc >= 2)
      strcpy(filnam, argv[1]);

    if(argc >= 3){
      if(strcmp(argv[2], "-both") == 0) BOTH = 'Y';
      else if(strcmp(argv[2], "-log") == 0) LOG = 'Y';
      else 
        strcpy(obj, argv[2]);
    }

    if(argc >= 4){
      if(strcmp(argv[3], "-both") == 0) BOTH = 'Y';
      if(strcmp(argv[3], "-log") == 0) LOG = 'Y';
    }

    if(argc == 5){
      if(strcmp(argv[4], "-both") == 0) BOTH = 'Y';
      if(strcmp(argv[4], "-log") == 0) LOG = 'Y';
    }

    if(LOG == 'Y') Dbg = 'Y';
    else Dbg = 'N';

/*    chkLength(filnam);  */

    fpin = fopen(filnam, "r");
    if(fpin == NULL) error("File not available(2)");
    fpout = fopen(obj, "w");
    if(fpout == NULL) error("File not available(3) OBJECTݕs");
    if(strstr(obj, "OBJECT")) fprintf(fpout, "[]\n");
    printf("\nEnglish program read from file:\n");	quo = 0;
//    sprintf(ejdic, "%sEJDIC.HON", dir);
    strcpy(ejdic, "EJDIC.HON");
//    ejd = fopen("EJDIC.HON", "r");
    ejd = fopen(ejdic, "r");
    setUp();
    zbClear();
    if(Dbg == 'Y'){ dbg = fopen("DEBUG.YAK", "w");
      wdr = fopen("WDREG", "w");
      fprintf(wdr, "{ꕶR[hS_JISw C-x RET f shift_jis\n");
    }
    strcpy(process, "PROCESS.YAK");
//    sprintf(process, "%sPROCESS.YAK", dir);
//    prcs = fopen("PROCESS.YAK", "w");
    if(LOG == 'Y') prcs = fopen(process, "w");
    mai1:
    printf("\n");
    // expression()̒l'1'(擪'<')̎
    if(expression() ){ srcUrl();  goto MAIN3; }
    if(egs[0] == '\0') goto MAI2;
    //if(buf[0] == '\0') goto MAI2;
    existence = 1;      /*    Sentence exist    */
    if(wnb < WordNumb-1) ajClear(wnb+1);
    qstMark();	yakDbg();	tekyk();	bunpo();
    siage(jps);
    // DEBUGt@C̃`FbN
    if(Dbg == 'Y'){
      dbgsz = ftell(dbg);
      if(dbgsz > 6000000){
	fprintf(dbg, "File Size is Over !!\n");
	fclose(dbg);
	Dbg = 'N';
      }
    }

    if(BOTH == 'Y') fprintf(fpout, "%s<BR> \n", egs);  // \̏ꍇ
  
  MAIN3:
    printf("\n");	puts(egs);
    printf("--------------------------------------------\n");
    puts(jps);	fprintf(fpout, "%s", jps);
    if(LOG == 'Y') fprintf(prcs, "%s \n", jps);
    if(quo == 0 && script == 0 && *jps != '<') fprintf(fpout, "\n");
    if(buf[0] == ';' || buf[0] == ',') goto mai1;
    goto mai1;
  MAI2:
    printf("End of file reached\n");
    if(LOG == 'Y') fclose(prcs);
    if(Dbg == 'Y'){ fclose(dbg); fclose(wdr);    }
    fclose(fpin);	fclose(fpout);	fclose(ejd);
    if(existence == 0){ printf("Sentence not exist!\n");  exit(1); }
    resave();
    exit(0);
}

srcUrl(){
    char *search = "_external.url.path : ";
    char *search_in = "_internal.url.path : ";
    char *pt, *pt1, *pt2, dir[2000];
    int  ln;

    if(strlen(path) == 0){
      if(strlen(path2) == 0){
 if(Dbg == 'Y') fprintf(dbg, "--- srcUrl path --- path = 0 \n");
      } else {
	strcpy(path, path2);
      }
    }
    strcpy(jps, egs);
    if(LOG == 'Y') fprintf(prcs, "--- srcUrl --- jps = %s \n", jps);
    if(pt = (char*)strstr(jps, search)){
      ln = strlen(search);
      wdcpy(path, pt + ln);
 if(Dbg == 'Y') fprintf(dbg, "--- srcUrl path --- path = %s \n", path);
      strcpy(path2, path);
      return;
    }
    if(strcmp(path, path2) != 0){
      if(Dbg == 'Y') fprintf(dbg, "--- srcUrl pathcmp --- path = %s path2 = %s \n", path, path2);
      strcpy(path, path2);
    }
    if(pt = (char*)strstr(jps, search_in)){
      ln = strlen(search_in);
      wdcpy(path_in, pt + ln);
      return;
    }
    if(strncmp(jps, "<!--", 4) == 0) return;
    if(strncmp(jps, "<![", 3) == 0) return;

//    if(strcmp(jps, "</SPAN>") == 0 || strcmp(jps, "</span>") == 0){
//      script = 0;    return;
//    }
    if(strcmp(jps, "</SCRIPT>") == 0 || strcmp(jps, "</script>") == 0){
      script = 0;    return;
    }
//    if(script == 1) return;
    if(strcmp(jps, "</STYLE>") == 0 || strcmp(jps, "</style>") == 0){
      style = 0;    return;
    }
    if((pt = (char*)strstr(jps, "@import")) ){
      if(pt1 = (char*)strstr(pt, "\"/")){ 
        ptreplace(pt1+1, 1, path);
        return;
      }
    }
    if(style == 1) return;

    if(pt = (char*)strstr(jps, "onUnload=\'")){
      if(pt1 = (char*)strchr(pt+10, '\'')){
        ptptcut(pt, pt1);
      }
    }
    if((pt = (char*)strstr(jps, "location.href"))){
      if(((pt1 = (char*)strstr(pt, "\"/")) || (pt1 = (char*)strstr(pt, "\'/"))) && pt1 < pt + 20){
        ptreplace(pt1+1, 1, path);
        return;
      }
    }
    if((pt = (char*)strstr(jps, ".adPopupFile"))){
      if(((pt1 = (char*)strstr(pt, "\"/")) || (pt1 = (char*)strstr(pt, "\'/"))) && pt1 < pt + 20){
        ptreplace(pt1+1, 1, path);
        return;
      }
    }
    if((pt = (char*)strstr(jps, "src=")) || (pt = (char*)strstr(jps, "SRC="))
    || (pt = (char*)strstr(jps, "BACKGROUND=")) || (pt = (char*)strstr(jps, "virtual="))){
      if(((pt1 = (char*)strstr(pt, "\"/")) || (pt1 = (char*)strstr(pt, "\'/"))) && pt1 < pt + 13){
        ptreplace(pt1+1, 1, path);
        return;
      }
      if(((pt1 = (char*)strstr(pt, "\"")) || (pt1 = (char*)strstr(pt, "\'"))) && pt1 < pt + 13){
        if(strncmp(pt1+1, "http:", 5)){
          ptinsert(pt1+1, path);
          return;
        } else {
          if(LOG == 'Y') fprintf(prcs, "--- srcUrl --- %s \n", jps);
          return;
        }
      }
      if(strncmp(pt+4, "http:", 5) == 0){
        return;
      }
      if((pt1 = (char*)strstr(pt, "/")) && pt1 < pt + 13){
        ptreplace(pt1, 1, path);
        return;
      }
    }

//    if(script == 1) return;
//    if(strncmp(jps, "<SPAN", 5) == 0 || strncmp(jps, "<span", 5) == 0){
//      script = 1;    return;
//    }
    if(strncmp(jps, "<SCRIPT", 7) == 0 || strncmp(jps, "<script", 7) == 0){
      script = 1;    return;
    }
    if(strncmp(jps, "<STYLE", 6) == 0 || strncmp(jps, "<style", 6) == 0){
      style = 1;    return;
    }

    if((pt = (char*)strstr(jps, "<base ")) || (pt = (char*)strstr(jps, "<BASE "))){
      if((pt1 = (char*)strstr(jps, ">"))){
        ptinsert(pt1+1, "-->");
        ptinsert(pt, "<!--");
        return;
      }
    }
    if(((pt = (char*)strstr(jps, "<meta ")) || (pt = (char*)strstr(jps, "<META "))) && (strstr(jps, "Refresh") || strstr(jps, "refresh"))){
      if((pt1 = (char*)strstr(jps, ">"))){
        ptinsert(pt1+1, "-->");
        ptinsert(pt, "<!--");
        return;
      }
    }
//    if((pt = (char*)strstr(jps, "<link href=")) || (pt = (char*)strstr(jps, "<LINK HREF="))){
//      if(((pt1 = (char*)strstr(pt, "\"/")) || (pt1 = (char*)strstr(pt, "\'/"))) && pt1 < pt + 13){
//        if(pt2 = (char*)strchr(pt1+2, '/')){
//          ptcpy(dir, pt1+2, pt2+1);
//          if(pt2 = (char*)strstr(path, dir)) *pt2 = '\0';
//        }
//        ptreplace(pt1+1, 1, path);
//        return;
//      }
//    }
    if((pt = (char*)strstr(jps, "<link "))    || (pt = (char*)strstr(jps, "<LINK "))){
      if((pt = (char*)strstr(jps, "href=")) || (pt = (char*)strstr(jps, "HREF="))){
        if(((pt1 = (char*)strstr(pt, "\"/")) && pt1 < pt + 13)
        || ((pt1 = (char*)strstr(pt, "\'/")) && pt1 < pt + 13)){
 if(Dbg == 'Y') fprintf(dbg, "--- srcURL <link href \"/ --- path = %s pt1 = %s \n", path, pt1);
          if(pt2 = (char*)strchr(pt1+2, '/')){
            ptcpy(dir, pt1+2, pt2+1);
            if(pt2 = (char*)strstr(path, dir)) *pt2 = '\0';
          }
          ptreplace(pt1+1, 1, path);
          return;
        }
        if(((pt1 = (char*)strchr(pt, '\"')) && pt1 < pt + 13)
        || ((pt1 = (char*)strchr(pt, '\'')) && pt1 < pt + 13)){
 if(Dbg == 'Y') fprintf(dbg, "--- srcURL <link href \" or \' --- path = %s pt1 = %s \n", path, pt1);
          if(strncmp(pt1+1, "http:", 5) == 0) return;
          if(pt2 = (char*)strchr(pt1+1, '/')){
            ptcpy(dir, pt1+1, pt2+1);
            if(pt2 = (char*)strstr(path, dir)) *pt2 = '\0';
          }
          ptinsert(pt1+1,  path);
          return;
        }
      }
      return;
    }

if((pt = (char*)strstr(jps, "<a ")) || (pt = (char*)strstr(jps, "<A "))){
    if((pt = (char*)strstr(jps, "href=")) || (pt = (char*)strstr(jps, "HREF=")) ){
      if(strncmp(pt+5, "http:", 5) == 0){
        ptinsert(pt+5, path_in);
        return;
      }
      if(((pt1 = (char*)strstr(pt, "\"/")) && pt1 < pt + 13)
      || ((pt1 = (char*)strstr(pt, "\'/")) && pt1 < pt + 13)){
 if(Dbg == 'Y') fprintf(dbg, "--- srcURL <a href \"/ --- path = %s path2 = %s pt1 = %s \n", path, path2, pt1);
        if(pt2 = (char*)strchr(pt1+2, '/')){
          ptcpy(dir, pt1+2, pt2+1);
//  printf("pt1+2 = %s dir = %s \n", pt1+2, dir);
          if(pt2 = (char*)strstr(path, dir)) *pt2 = '\0';
        }
        ptreplace(pt1+1, 1, path);
        ptinsert(pt1+1, path_in);
        return;
      }
      if((pt1 = (char*)strstr(pt, "\"")) && pt1 < pt + 7){
        if(strncmp(pt1+1, "http:", 5) == 0){
          ptinsert(pt1+1, path_in);
          return;
        }
        else {
          ptinsert(pt1+1, path);
          ptinsert(pt1+1, path_in);
          return;
        }
      }
      if(strncmp(pt+5, "/", 1) == 0){
        pt1 = pt+5;
        if(pt2 = (char*)strchr(pt1+1, '/')){
          ptcpy(dir, pt1+1, pt2+1);
          if(pt2 = (char*)strstr(path, dir)) *pt2 = '\0';
        }
        ptreplace(pt1, 1, path);
        ptinsert(pt1, path_in);
        return;
      }
      else {
        pt1 = pt + 5;
        ptinsert(pt1, path);
        ptinsert(pt1, path_in);
        return;
      }
    }
}
    if((pt = (char*)strstr(jps, "charset=")) || (pt = (char*)strstr(jps, "CHARSET=")) ){
      if(pt1 = (char*)strstr(pt, "windows-1252")){
        ptreplace(pt1, 12, "ISO-20220-JP");
      }
      if((pt1 = (char*)strstr(pt, "iso-8859-1")) || (pt1 = (char*)strstr(pt, "ISO-8859-1"))){
        ptreplace(pt1, 10, "ISO-20220-JP");
      }
      if(pt1 = (char*)strstr(pt, "UTF-8")){
        ptreplace(pt1, 5, "ISO-20220-JP");
      }
    }
    if((pt = (char*)strstr(jps, "lang=")) || (pt = (char*)strstr(jps, "LANG=")) ){
      if(pt1 = (char*)strstr(pt, "en-US")){
        ptreplace(pt1, 5, "ja");
      } else if(pt1 = (char*)strstr(pt, "en")){
        ptreplace(pt1, 2, "ja");
      }
    }
}

nextword(){
    int  ch = 0, i = 0;
    int  j;

    if(arrw == 1){ buf[i++] = '<'; arrw = 0; }
    while((char) 1){
      ch = getc(fpin);	if(ch == EOF) break;
      // |󕶂̏ꍇ
      if((html_f == 0 && script == 0) && (ch == ' '|| ch == '\n' || ch == 13 || ch == '\t')){
        if(i == 0) continue;
        else break;
      }
      buf[i++] = ch;  buf[i] = '\0';

      // HTML <!--  -->Ή CMNT:RgtO   03/04/10
      if(ch == '>'){
        if(CMNT != 0){
          if(strstr(buf, "-->") || strstr(buf, "]>")){ CMNT = 0; html_f = 0; break; }
          else continue;
        } else
        if(html_f != 0){ html_f = 0;    break; }
      }
      if(ch == '<'){
//        if(strncmp(buf, "<![", 3) == 0){ CMNT = 1;  continue; }
        arrw = 1;    html_f = 1;
        break;
      }
      if(ch == '-'){
        if(strncmp(buf, "<!--", 4) == 0){ CMNT = 1;  continue; }
      }

      if(html_f != 0 || script != 0 || style != 0 || CMNT != 0) continue;

      if(ch == '[' || ch == '(') break;
      //      if(ch == '[' || ch == ']' || ch == '(') break;
      if(strcmp(buf, "g") == 0 || strcmp(buf, "e") == 0) break;
      if(ch == '?' || ch == '!' || ch == ';') break;
      if(ch == '\'' && i == 1) break;
      if((strstr(buf, ".com") || strstr(buf, ".jp") || strstr(buf, ".gov")) && !strstr(buf, "http://")) break;
    }
    buf[i] = '\0';

    if(buf[strlen(buf)-1] != '.' && buf[0] != '<' && script == 0 && style == 0){
      for(j = 0; j < i; j++){
        if(strchr(buf, '-') != 0 || strchr(buf, '/') !=0) break;
        // 啶ɕϊ
        // ϊȂ悤ɂ 
//        if(buf[0] != '<' && html_f == 0 && i > 2 && j > 0 && isupper(buf[j]) && buf[i-1] != '(') buf[j] = tolower(buf[j]);
//        if(buf[i-1] == '<' && j > 0 && isupper(buf[j])) buf[j] = tolower(buf[j]);
      }
    }
    // Pꂪ啶̏ꍇďϊ
   if(html_f == 0 && script == 0 && isupper(buf[1]) ) StrToLower(buf+1);

 if(Dbg == 'Y') fprintf(dbg, "--- nextword --- buf = %s html_f = %d script = %d \n", buf, html_f, script);
}
  

error(str)
char *str;
{
    printf("\n %s\n", str);	exit(1);
}

expression(){
    int  loop;
    int  i = 0, ln, ppt, cs;
    char *ptc, c;
    buf[0] = '\0';	egs[0] = '\0';	wc = -1;
    wdx[0] = '\0';	wd[0] = '\0';
    ajClear(0);
    for(i = 0; ; i++){
      nextword();		/* t@CPǂݍ */
      //if(buf[0] == '\0') return(0);
      if(buf[0] == '\0') break;
      ln = strlen(buf);
      c = buf[strlen(buf)-1];
      if(c == '<'){
        buf[strlen(buf)-1] = '\0';
        if(buf[0] == '\0' && *egs == '\0') continue;
        if(buf[strlen(buf)-1] == '.') c = '.';
      }
      strcat(egs, buf);
      //      if(strncmp(buf, "&#", 2) == 0) break;
//   fprintf(dbg, "--- exp --- egs = %s script = %d \n", egs, script);
      // html֐̃`FbN =1
      if(*buf == '<' || script != 0 || strstr(buf, "---")) return(1);

      if(buf[0] != '&' || buf[strlen(buf)-1] != ';') strcat(egs, " ");
//   fprintf(dbg, "--- exp --- egs = %s script = %d \n", egs, script);

      if(strcmp(buf+strlen(buf)-2, ".\"") == 0) break; // buf : xxxx." 15/08/08
      if(strcmp(buf+strlen(buf)-3, ".h") == 0) break; // buf : xxxx." 15/11/02
      if(c == '<') break;
      if(c == '!' || c == '?' || c == ']') break;
      if(c == ':' && isdigit(buf[strlen(buf)-2])==0) break;
      if(c == '.'){
        if(strstr(buf, "htm")) return(1);
        if(strchr(buf, '@') || strchr(buf, '/')) continue;
//        if(buf[strlen(buf)-2] >= '0' && buf[strlen(buf)-2] <= '9') continue;
          if(strlen(buf) == 2 && isupper(buf[strlen(buf)-2])) continue; // 
        // 
        if(strcmp(buf, "Jan.") == 0 || strcmp(buf, "Feb.") == 0) continue;
        if(strcmp(buf, "Mar.") == 0 || strcmp(buf, "Apr.") == 0) continue;
        if(strcmp(buf, "Jul.") == 0 || strcmp(buf, "Aug.") == 0) continue;
        if(strcmp(buf, "Sept.") == 0 || strcmp(buf, "Oct.") == 0) continue;
        if(strcmp(buf, "Nov.") == 0 || strcmp(buf, "Dec.") == 0) continue;
        // j
        if(strcmp(buf, "Mon.") == 0 || strcmp(buf, "Wed.") == 0) continue;
        if(strcmp(buf, "Tue.") == 0 || strcmp(buf, "Tues.") == 0) continue;
        if(strcmp(buf, "Thur.") == 0 || strcmp(buf, "Th.") == 0) continue;
        if(strcmp(buf, "Fri.") == 0 ) continue;
        if(strcmp(buf, "Sun.") == 0 || strcmp(buf, "Sat.") == 0) continue;
         
	if(strcmp(buf,"Corp.") == 0 || strcmp(buf, "Inc.") == 0) continue;
        if(strcmp(buf, "Co.") == 0 || strcmp(buf, "Ltd.") == 0) continue;
        if(strcmp(buf, "Capt.") == 0 || strcmp(buf, "Gov.") == 0) continue;
	if(strcmp(buf, "vs.") == 0) continue;
        if(strcmp(buf, "tel.") == 0) continue;
	if(strcmp(buf, "etc.") == 0 || strcmp(buf, "pct.") == 0) continue;
        if(strcmp(buf, "ft.") == 0 || strcmp(buf, "Sgt.") == 0) continue;
	if(strcmp(buf+ln-4,"vol.")== 0 || strcmp(buf+ln-3, "No.") == 0) continue;
        if(strcmp(buf,"e.") == 0 || strcmp(buf, "e.g.") == 0) continue;
        // E
        if(strcmp(buf, "Lt.") == 0 || strcmp(buf, "Col.") == 0) continue;
        // ꏊ
        if(strcmp(buf, "Govt.") == 0 || strcmp(buf, "Okla.") == 0) continue;
        if(strcmp(buf, "Colo.") == 0 || strcmp(buf, "Nev.") == 0) continue;
        if(strcmp(buf, "Wash.") == 0 || strcmp(buf, "Va.") == 0) continue;
        if(strcmp(buf, "Calif.") == 0) continue;
        // 
	if(strcmp(buf, "Japan-U.S.") == 0) continue;
        if(strcmp(buf,"U.") == 0 || strcmp(buf, "U.S.") == 0
        || strcmp(buf, "U.K.") == 0 || strcmp(buf, "U.N.") == 0) continue;
        if(strcmp(buf, "N.") == 0 || strcmp(buf, "N.Y.") == 0
        || strcmp(buf, "N.J.") == 0 || strcmp(buf, "N.C.") == 0) continue;
        if(strcmp(buf, "L.") == 0 || strcmp(buf, "L.A.") == 0) continue;
        if(strcmp(buf, "St.") == 0) continue;
        if(strcmp(buf,"a.") == 0 || strcmp(buf, "a.m.") == 0) continue;
        if(strcmp(buf,"p.") == 0 || strcmp(buf, "p.m.") == 0) continue;
        if(strcmp(buf,"P.") == 0 || strcmp(buf, "P.M.") == 0) continue;
        if(strcmp(buf, "www.") == 0) continue;
        // 
        if(strcmp(buf, "Ms.") == 0 || strcmp(buf, "Dr.") == 0) continue;
        if(strcmp(buf, "Mr.") == 0 || strcmp(buf, "Mrs.") == 0) continue;
        if(strcmp(buf, "Jr.") == 0 || strcmp(buf, "Maj.") == 0) continue;
        if(strcmp(buf, "Gen.") == 0 || strcmp(buf, "Cpl.") == 0) continue;
        if(strcmp(buf, "Sen.") == 0 || strcmp(buf, "Sens.") == 0) continue;
        if(strcmp(buf, "Rep.") == 0 || strcmp(buf, "Spc.") == 0) continue;
        if(strcmp(buf, "Adm.") == 0) continue;
        // n
        if(strcmp(buf, "Mt.") == 0 || strcmp(buf, "Pfc.") == 0) continue;
        if(strcmp(buf, "Rev.") == 0) continue;
        //
        if(strcmp(buf, "S.L.U.T.") == 0) continue;
        if(strcmp(buf, "A.S.A.P.") == 0) continue;
        if(strcmp(buf, "O.J.") == 0) continue;
        // ̑̓yIh𗣂
        insertc(egs, strlen(egs)-2, ' ', strlen(egs));
        break;
      }
    }
    if(egs[0] == '\0') return(0);
    if(ptc = strstr(egs, "&#")){
      if(strncmp(ptc, "&#8212;", 7)) return(1); // em_bVȊO̎
    }
    if( strstr(egs, "&bull;") || strstr(egs, "&raquo;")) return(1);
    if((ptc = (char*)strchr(egs, '%')) && !isdigit(*(ptc-1))) return(1);
    if((ptc = (char*)strstr(egs, "@import")) ) return(1);
    // style Lēx̗p 20/8/16
    if(style != 0) return(1);
//    if((ptc = (char*)strchr(egs, '@')) && isalnum(*(ptc+1))) return(1);

    if(strlen(egs) >= SentLen-4){
      printf("\n Too long sentence !! (%d) bytes over !! \n", strlen(egs)-SentLen+4);
//      printf(" p܂H  [Y/N]= ");
//      while(c != 'y' && c != 'n') c = tolower(getchar());
//      if(c != 'y') return(1);
    }
    wnb = i;	strcpy(wdh, egs);
//  printf("--- exp --- egs = %s \n", egs);
    predit(wdh, &no);
    // wdhsp݂̂̏ꍇ
    if(strcmp(wdh, " ") == 0) return(1);
    wc = -1;
//  printf("--- exp --- \n%s \n\n", wdh);
    wdcpy(wd, wdh);
    for(wi = 0; ; wi++){
      if(wdh[0] == '\0') break;
      strcpy(wp, wd);
// printf("--- expression2 --- wd = %s \n", wd);
      if(*wd <= 'Z' && *wd >= 'A') Cap = 'O';	else Cap = '\0';
      cnb = 0;    cs = 1;
      symbck();
      if(ct == 1){ strcpy(wdw, wdh);	strcpy(wdp, wd);
	strcpy(wdx, wd);
        kakut(&cs);
        continue;
	//goto ex1;
      }
      zkc = '\0';	cnb = -1;
      loop = 15;
      for(wdx[0] = '\0'; wdx[0] == '\0'; ){
	if(zkc != '\0' && zkc != '-'){
	  ln = strlen(wdp);
	  if((ptc = (char*)strstr(wdh, wdp)) && ptc == wdh){
	    if(*(wdh+ln) == ' ' || *(wdh+ln) == '\0'){
	      strcpy(wdw, wd);	strcat(wdw, wdh+ln);
	    }
	    else{
	      strcpy(wdw, ptc+ln);	goto CONJG;
	    }
	  }
	  else{
	    CONJG:
	    if(ln < 2) ln = 2;
	    strcpy(wdw, wd);
	    if(ptc = (char*)index(wdh+ln-2, ' ')) strcat(wdw, ptc);
	  }
	}
	else strcpy(wdw, wdh);
//  printf("--- expression1 --- wc = %d  wdp = %s  wdw = %s  wdh = %s \n",wc, wdp,wdw,wdh);
//        if(*wdw == '<') return(1);
	ppt = ptcal(dics,dic,wdw);
        if(ppt == 0) break;
	diclook(ppt, &cs);
	if(wdx[0] == '\0') jishoFail(&cs);
      //ex1:
 if(Dbg == 'Y') fprintf(dbg, "--- expression --- wd = %s  wdp = %s  ppt = %d \n", wd, wdp, ppt);
	kakut(&cs);
        if(cs == -1) return(-1);
        // [v񐔃I[o[I
        if(--loop == 0){
   if(Dbg == 'Y') fprintf(dbg, "--- exppression --- [v񐔃I[o[ egs = %s cs = %d \n", egs, cs);
          Cmp = '\0';  return(-1);
        }
      }
    }
    wnb = wc;
    return(0);
}


/*----------------------------------------------------------------------*/

setUp(){
    if(LOG == 'Y') Dbg = 'Y';
    else Dbg = 'N';
}

jishoFail(cs)
int  *cs;
{
    int  jr;
/*    if(caj[0][0] != '\0' && zkc == 'x'){
      strcpy(wdx, wd);	strcpy(wdw,wdh);	return;
    }*/
    Conj(wdp, wdh, cs);
 if(Dbg == 'Y') fprintf(dbg, "--- jishoFail --- wdh = %s *cs = %d  zkc = %c \n", wdh, *cs, zkc);
    if(*cs == 9)
      if(Dbg == 'Y') fprintf(wdr, "%s:::\n", wd); // 16/5/22

    if(zkc == '\0' || (zkc == 'K' && wc == -1)){
      engcopy();
	strcpy(wdx, wd);	strcpy(wdw, wdh);
      if(instr(wdw, wd) != 1){
	jr = instrc(wdw, ' ');	left(wd, wdw, jr-1);
      }
      cnb = 2;
    }
//    else{
      // bug 邩H preConjɉe
//      if(*wdh < 90 && !strchr(wdp, '\'')){
//        *wdh += 32;  *wdp += 32; strcpy(wd, wdp);
//      }
//    }
    if(zkc == 'Y') zkc = '\0';
}

kakut(cs)
int  *cs;
{
    int  i, ln, kakf = 0;
    char kd[WordLen], *gobi, cac, *ptc;
    cac = caj[0][0];
 if(Dbg == 'Y') fprintf(dbg, "--- kakut --- caj[0] = %s  zkc = %c \n", caj[0], zkc);
    if(cac == '\0'){
 if(Dbg == 'Y') fprintf(dbg, "--- kakut --- caj[0] = %s  zkc = %c \n", caj[0], zkc);	return;
    }
    ct = 0;
    if(cac == '0' || cac == '?' || cac == '!' || cac == '|'){
      wc++;	strcpy(kd, wd);	kakf = 1;	goto SAIYOU;
    }
    if((!index(wd, ' ')) && (cac == '1' || cac == '3' || cac == '5' ||
    cac == '2' || cac == '7')){
/*      if(zkc == '\0' || zkc == 'O' || (zkc != 'K' && zkc != 'h' && zkc != 'G')){*/
      if(zkc == '\0' || zkc == 'O' || (zkc != 'K' && zkc != 'h' && zkc != 's')){
	find_root();
	if(ct == 1){ wdx[0] = '\0';	ct = 0;	return; }
      }
    }

    wc++;
    if(zkc == 'h' || zkc == 's'){
 if(Dbg == 'Y') fprintf(dbg, "--- kakut --- wdp = %s  zkc = %c \n", wdp, zkc);
      ln = strlen(wdp);
      if(wdw[ln]!=' ' && wdw[ln+2]==' ' && (wdw[ln+1]=='s' || wdw[ln]=='s'))
        left(wdp,wdw, ln+2);
      strcpy(wd, wdp);
      if(index(caz[0], 'h')) zkc = 'h';
      posses();
      if((! *wdx) && *wdp) strcpy(wdx, wdp);
      zkc = '\0';
    }
    if(Cmp == '-'){ Cmp = '\0';
      compoWord();	strcpy(kd,wd);	ln = strlen(kd)-2;	goto NXTWD;
    }
    if(zkc == 'n'){
    }
    if(zkc == 'K' && *wdx == '\0'){
      if(*cs == 1 && *wd >= 'a'){ wc--;	return; }
      cnb = 0;

      if((! strstr(wdw, wd)) && (wdh == (char*)strstr(wdh, wdp)))
	strcpy(wdw, wdh);
/*      if(*wd >= 'a' && *wd <= 'z'){
	caj[1][0] = '\0';
	if(caj[0][0] == '5'){
	  ln = strlen(caj[0]);	gobi = caj[0]+ln-2;
	  if(strcmp(gobi, "") == 0 || strcmp(gobi, "") == 0) *gobi = '\0';
	}
      }
      else
	if(! *wdx) strcpy(wdx, wdp);*/
      strcpy(wdx, wdp);
    }
    if(zkc == 'l'){ lykakut(wdp);
      if(caj[0][0] == '\0'){ strcat(wdp, "ly");	*wdx = '\0';	return; }
    }
    if(wc >= 1 && aj[wc-1][0][0] == '\0'){
    }
    if(wc >= 1){
      if(strcmp(wdp,"her")==0 && aj[wc-1][0][0]=='#' &&
      (strcmp(wdw,wdp)==0 || instr(wdw, "her #")==1))
	herCk();
    }
    if(index(wdx, '#')) strcpy(kd, wdx);
    else {
      if(*wdp != '\0' && (char*)strstr(wdw, wdp) == wdw) strcpy(kd, wdp);
      else strcpy(kd, wd);
    }
 // if(Dbg == 'Y') fprintf(dbg, "--- kakut --- wdh = %s  wdp = %s  wdw = %s \n",wdh,wdp, wdw);
    umeKakut(kd, wdw);
    SAIYOU:
    if(index(wdx, '#') || *wdw == '\0') strcpy(en[wc], wdx);
    else strcpy(en[wc], wdp);
    strcpy(pwd, en[wc]);
    if(cnb < 5){
      aj[wc][cnb+1][0] = '\0';	az[wc][cnb+1][0] = '\0';
    }
    if(Cap && zkc != 'O')
      for(i = 0; i <= cnb; i++){ strcpy(aj[wc][i], caj[i]);
	strcpy(az[wc][i], "O");	strcat(az[wc][i], caz[i]);
      }
    else
      for(i = 0; i <= cnb; i++){
	strcpy(aj[wc][i], caj[i]);	strcpy(az[wc][i], caz[i]);
      }
    invar_vb();
    ln = strlen(kd)-2;
    if(ln > 0 && (*(wdw+ln) == ' ' || *(wdw+ln) == '\0')){
      gobi = kd+ln;
      if(strcmp(gobi, "'s") == 0 || strcmp(gobi, "s'") == 0){
	*gobi = '\0';	ln -= 2;
      }
    }
    NXTWD:
    if(*kd == (*wdw+32)) *kd -= 32;
    if(strcmp(kd, wdw) < 0){
      if(wc >= WordNumb){
	printf("\nP̐ (%d) \n", wc);  *cs = -1;  return;
	exit(1);
      }
      if(Cmp == '~'){
	ptcut(wdh, ln+3);	Cmp = '-';
      }
      else{

	right(wdw, wdh, strlen(wdw)-ln-2);	zkc = '\0';
      }
      if(*wdh == '\'' && *(wdh+1) == 's') ptcut(wdh, 3);
      if(*wdh == 's' && *(wdh+1) == '\'') ptcut(wdh, 3);
      while(*wdh == ' ') ptcut(wdh, 1);
      *wdp = '\0';
      if((ptc = (char*)index(wdh, ' ')) || (ptc = (char*)index(wdh, '\0')))
	ptcpy(wd, wdh, ptc);
      if(index(wdh, ' ') == 0) wdcpy(wd, wdh);
 if(Dbg == 'Y') fprintf(dbg, "--- kakut --- wdh = %s \n", wdh);
      if(kakf != 0) strcpy(wdx, wd);
      return;
    }
    if(strcmp(kd, wdw) == 0){ *wdh = '\0';	*wdp = '\0'; }
    else{
      printf("wdw == error!!  kd = %s  wdw = %s  wd = %s \n",kd,wdw,wd);
      printf("pP%s̊p^͂̕Ɏs܂B", wd);
      exit(1);
    }
}

find_root(){
 if(Dbg == 'Y') fprintf(dbg, "--- find-root --- wdx = %s  zkc = %c  wd = %s \n", wdx,zkc,wd);
    ct = 0;
    /*  ones, using, used ̏ꍇ  */
    if((zkc == 'x' && (caj[0][0] != '1' && caj[0][0] != '3')) ||
    ((zkc == 'G' || zkc == 'p') && caj[0][0] != '3')){
      wdcpy(wd, wdh);	strcpy(wdp, wd);	wdx[0] = '\0';
    }
    if(wdx[0] != '\0') return;
    ct = 1;
    if(zkc != '\0') return;
    if(zkc == 'O'){ zkc = '\0';	return; }
    ct = 1;	zkc = 'p';
}

compoWord(){
    int  ln1;
    ln1 = strlen(aj[wc-1][0]);
    if(index(az[wc-1][0], 'n') && ln1 == 5 &&
      ninstr(ln1-1,aj[wc-1][0],"O") == ln1-1){
      strcpy(aj[wc-1][0]+3, caj[0]+1);
/*      strcpy(aj[wc-1][1]+3,caj[1]+1);*/
      strcpy(aj[wc-1][1]+3, caj[0]+1);
    }
    else{
      aj[wc-1][0][0] = caj[0][0];
      strcat(aj[wc-1][0],caj[0]+1);	strcat(az[wc-1][0],caz[0]);
      if(aj[wc-1][1][0] != '\0'){
	aj[wc-1][1][0] = caj[0][0];
	strcat(aj[wc-1][1],caj[0]+1);	strcat(az[wc-1][1],caz[0]);
      }
    }
    strcat(en[wc-1], "-");	strcat(en[wc-1],wd);	zkc= Cmp= '\0';	wc--;
}

dollCk(wdp)
char wdp[];
{
    int  jz, wc1, jz1 = 0, pl = 0;
    char z, dums[5], azd[10], az1[10];
    ct = 1;	wc1 = wc-1;
    strcpy(azd, az[wc1][0]);	jz = instrc(azd, '$');
    strcpy(az1, az[wc1][1]);
    if(*az1) jz1 = instrc(az1, '$');
    if((jz && jz1 == 0 && aj[wc1][1][0]) || (jz == 0 && jz1)) pl = 1;
    if(jz == 0 && jz1){ jz = jz1;	strcpy(azd, az1); }
    if(jz && aj[wc1][2][0] && (! index(az[wc1][2], '$'))) pl = 2;
    if(jz > 0 && (strlen(azd) == 3 || strlen(azd) == 4)){
      z = azd[++jz];
      if(z == 's'){
	dums[0] = '\0';
	if(strlen(wdp) >= 4) right(wdp, dums, 4);
      }
    }
}

herCk(){
    int  jz;
    char bzd[10];
    strcpy(bzd, az[wc-1][0]);
    jz = instrc(bzd, '#');
}

lykakut(wdp)
char wdp[];
{
    int  i, j, j1, ln;
    char iid[4];
    for(i = 0; i <= cnb; i++){
      if(caj[i][0] == '\0') break;
      if(caj[i][0] != '5'){
	for(j = i; j < cnb; j++){
	  j1 = j+1;
	  strcpy(caj[j], caj[j1]);	strcpy(caz[j], caz[j1]);
	}
	caj[cnb][0] = '\0';	caz[cnb--][0] = '\0';
      }
    }
    if(caj[0][0] == '\0'){
      zkc = '\0';	return;
    }
    strcpy(wd, wdp);
    for(i = 0; i <= cnb; i++){
      if(caj[i][0]){
	ln = strlen(caj[i])-2;	right(caj[i], iid, 2);
	if(strcmp(iid, "")==0 || strcmp(iid, "")==0){
	  replaces(caj[i], ln, 2, "");
	}
	if(strcmp(iid, "")==0){
	  replaces(caj[i], ln, 2, "");
	}
	if(strcmp(iid, "")==0) replaces(caj[i], ln, 2, "");
	caj[i][0] = '6';	ptcut(caz[i], 1);
      }
      else{
	cnb = i-1;	break;
      }
    }
}

umeKakut(kd, wdw)
char kd[], wdw[];
{
    int  i, j1, n2, ln, ln1, h2, h3;
    char k1[45], k2[45], k3[40], k2d[40], hd[2], dmy[2], *ptj, nbstr[3];
    j1 = instrc(kd, '#');
    while(j1 >= 2){
 if(Dbg == 'Y') fprintf(dbg, "--- umeKakut -- kd = %s wdw = %s \n",kd,wdw);
      left(k1, kd, j1-2);
      n2 = ninstrc(j1, kd, ' ');
      ln = strlen(kd);	right(kd, k2, ln-n2);
      if(kd[j1] == ' ') hd[0] = '\0';
      else{ hd[0] = kd[j1];	hd[1] = '\0'; }
      if(kd[ln-1] == '>') cutoff(k2, strlen(k2)-2, 2);
      h3 = instrc(k2, '#');	ln1 = strlen(k1);
      if(h3 == 0) h2 = ninstr(ln1+2, wdw, k2);
      else{ left(k2d, k2,h3-2);	h2 = ninstr(ln1+2, wdw, k2d); }
      mid(wdw, ln1+1, h2-ln1-2, k3);
      cutoff(wdw, 0, h2-1);
      no++;
      numbtohan(no, nbstr);
      for(i = 0; i <= YakugoMax; i++){
	if(!caj[i][0]) break;
	if(ptj = (char*)index(caj[i], '#')){
/*	  if(isdigit(*(ptj+1)) != 0) ptj = (char*)index(ptj+2, '#');*/
	  if(*(ptj+1) >= 0x30 && *(ptj+1) <= 0x39)
	    ptj = (char*)index(ptj+2,'#');
	  if(ptj) ptinsert(++ptj, nbstr);
 if(Dbg == 'Y') fprintf(dbg, "--- umeKakut -- caj[%d] = %s \n",i,caj[i]);
	}
      }
      strcpy(kd, k2);	j1 = instrc(kd, '#');
      *dmy = no+'0';	dmy[1] = '\0';
      strink(wdw, " #", dmy, hd, k3, (char*)0);
 if(Dbg == 'Y') fprintf(dbg, "--- umeKakut --- %s  wdw = %s  k3 = %s \n", kd, wdw, k3);
    }
}

invar_vb(){
    char *ptc;
    if(caj[0][0] == '3' && (caj[1][0] == '\0' || caj[2][0] == '\0') &&
    (ptc = (char*)index(en[wc], ' '))){
    }
}

ajClear(t)
int  t;
{
    int  i, j;
    for(i = t; i <= WordNumb-1; i++){
      for(j = 0; j <= YakMaxCt; j++){
	aj[i][j][0] = '\0';	az[i][j][0] = '\0';
      }
      en[i][0] = '\0';
    }
}

zbClear()
{
    sj[0] = '\0';	oj[0] = '\0';	vr[0] = '\0';
    sz[0] = '\0';	oz[0] = '\0';	vz[0] = '\0';	pst[0] = '\0';
}

posses()
{
    cutoff(caj[0], 0,1);	strcat(caj[0], "");
    if(zkc == 'h' || index(caj[0], 'a')){
      strcpy(caj[1], caj[0]);	insertc(caj[1],0,'1',strlen(caj[1]));
      strcpy(caz[1], caz[0]);	cnb = 1;
    }
    else{
      caj[1][0] = '\0';	cnb = 0;
    }
    insertc(caj[0], 0, '8', strlen(caj[0]));
}

qstMark()
{
    int  i, j, cn;
    for(i = wnb; i >= 1; i--)
      if(aj[i][0][0] == '?'){
	for(j = i; j >= 0; j--){
	  // if(strcmp(aj[j][0], "|h") == 0) return;
	  if(j && aj[j][0][0] == '0' && aj[j-1][0][0] == 'X') return;
	  if(en[j][0] == '.') return;
	  for(cn = 0; cn <= YakMaxCt; cn++){
	    if(aj[j][cn][0] == '\0') break;
	    else insertc(az[j][cn], 0, '?', strlen(az[j][cn]));
	  }
	}
      }
}

yakDbg()
{
    int  i, j;

    for(i = 0; i <= wnb; i++){
      if(LOG == 'Y') fprintf(prcs, "\n");
      for(j = 0; j <= YakMaxCt; j++){
	if(aj[i][j][0] == '\0') break;
	  if(LOG == 'Y') fprintf(prcs, "aj[%d][%d] = %s     ", i, j, aj[i][j]);
	  if(LOG == 'Y') fprintf(prcs, "az[%d][%d] = %s   \n", i, j, az[i][j]);
      }
    }
}

yakAkitume(){
    int  i, j, ct;
    for(i = 0; i <= cnb; i++){
      if(caj[i][0] == '\0') break;
    }
    if(i > cnb) return;
    ct = i-1;
    if(i == cnb){ cnb--;	return; }
    for(j = i+1; j <= cnb; j++)
      if(caj[j][0] != '\0'){ ct++;
	strcpy(caj[ct], caj[j]);	strcpy(caz[ct], caz[j]);
      }
    cnb = ct;	caj[cnb+1][0] = '\0';
}

chkLength(file)
char *file;
{
    int  i = 0;
    char c, command[30];
    chkL0:	fpin = fopen(file, "r");
    if(fpin == NULL) error("File not available(1)");
    chkL1:	buf[0] = '\0';	egs[0] = '\0';
    for(i = 0; ; i++){
      nextword();	if(buf[0] == '\0'){ fclose(fpin);	return; }
      strcat(egs, buf);	strcat(egs, " ");
      c = buf[strlen(buf)-1];
      if(c == '!' || c == '?') break;
      if(c == '.' && strcmp(buf, "Mr.")!= 0 && strcmp(buf, "Mrs.")!= 0 &&
        strcmp(buf, "Mt.")!= 0 && strcmp(buf, "Dr.")!= 0 &&
	strcmp(buf,"e.g.")!= 0 && strcmp(buf, "U.S.")!= 0) break;
    }
    if(strlen(egs) >= SentLen-4){
      printf("\n %s \n\n (", egs);	color_light_blue;
      printf("%d", strlen(egs)-SentLen+4);	color_white;
      printf(") bytes long over !! \n");
      printf(" p܂H  [Y/N]= ");
      c = '\0';	while(c != 'y' && c != 'n') c = tolower(getchar());
      if(c != 'y'){
        fclose(fpin);	printf(" Emacsŕ͂܂H  [Y/N]= ");
        c = '\0';	while(c != 'y' && c != 'n') c = tolower(getchar());
	if(c != 'y') exit(1);
/*	execall("ME.EXE", file);	goto chkL0;
	sprintf(command, "A:EM98.EXE %s", file);
	system(command);	goto chkL0;*/
      }
    }
    if(i > 35){
    }
    goto chkL1;
}
